.ert-label {
    white-space: nowrap;
}
.ert-plot-options {
    height: 84px !important;
}
.ert-parameter-selector-container-hide {
    display: none;
}
.ert-parameter-selector-container-show {
    display: block;
}
.ert-parameter-label-checkbox {
    display: block;
}
.ert-input-number {
    width: 65px;
}
.ert-parameter-view-caption {
    width: auto;
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
}
.ert-parameter-view-caption-tooltip {
    background-color: rgba(56,108,176, 0.8);
    color: white;
    padding-left: 20px;
    padding-right: 20px;
    padding-top: 10px;
    padding-bottom: 10px;
}

.selected-ensemble-dropdown> div > div >div.Select-value:nth-child(5n-4) {
  background-color: rgba(56,108,176,0.8);
  color: black;
}

.selected-ensemble-dropdown> div > div >div.Select-value:nth-child(5n-3) {
  background-color: rgba(127,201,127,0.8);
  color: black;
}

.selected-ensemble-dropdown> div > div >div.Select-value:nth-child(5n-2) {
  background-color: rgba(253,192,134,0.8);
  color: black;
}

.selected-ensemble-dropdown> div > div >div.Select-value:nth-child(5n-1) {
  background-color: rgba(240,2,127,0.8);
  color: black;
}

.selected-ensemble-dropdown> div > div >div.Select-value:nth-child(5n-0) {
  background-color: rgba(191,91,23,0.8);
  color: black;
}
